subkw_parse_store

Function subkw_parse_store 

fn subkw_parse_store<KO>(
    subkw: &Ident,
    input: ParseStream<'_>,
    dest: &mut Option<Argument<KO>>,
) -> Result<()>
Helper for impl_parse_subkeywords!; parse and store subkw argument

Parses and stores a template element argument to a subkeyword, using Argument::parse.

Detects repeated specification of the same keyword, as an error.

KO is the lexical parsing context, and determines what kind of values the template author can supply.